//Datei graphalg1.sci
//
//Beispiel zur Anwendung des implementierten Algorithmus

// Zuerst in SciPad Execute->Load into Scilab wählen dann:

// 1. in Scilab g=einfacher_graph;
// oder
// g=kleiner_graph(); eingeben (mit Semikolon);.
// 2. show_graph(g)
// 3. im Graphenfenster Menu->Options w?hlen
// 4. Anzeige von Node: Name auswählen
// 5. in Scilab: g=top_sort(g);
// 6. show_graph(g)
// 7. der Graph mit den neuen Knotennummern(-namen) wird angezeigt

// Die Funktion kleiner_graph erzeugt einen Graphen mit 8 Knoten, 
//die Funktion einfacher_graph einen mit 4 Knoten.
//Graph1 und Graph2 haben diesselbe Anzahl von Knoten, Graph2 enthält Kanten <i,j> und <j,i>



///////////////////////////////////////////////////
//**********************************************///
///////////////////////////////////////////////////

function [g]=top_sort(graph)
//sortiert einen Graphen graph topologisch
//die interne nummerierung bleibt erhalten,
//nur die Namen der Knoten werden ge?ndert.
//um ?ber diese den n?chsten Knoten zu erhalten
//ist die Funktion get_next_node aufzurufen.


//Eingabedaten:
mat=graph_2_mat(graph,'node-node'); //Adjanzenzmatrix

n=node_number(graph); //Anzahl der Knoten

// Schritt 1:

v=1; //neue Knotennummer
N=ones(n,1); //Menge der noch nicht sortierten Knoten, als Vektor mit 1=noch nicht sortiert, 0=sortiert

vorg=zeros(n); //Vorg?nger
vorg=sum(mat,'r'); //vorg(j)=summe (aij) f?r alle j aus N

nodenamesnew=graph.node_name; //Neuen Knotennamen

weiter=1;
while weiter>0,
k=1; //Schritt 2:
weiter2=1;
while weiter2>0, //finde ein k aus N mit keinem vorg?nger
if N(k)==0 then,
k=k+1;
else,
if vorg(k)==0 then, //Schritt 3:
nodenamesnew(k)=msprintf('%d',v); //setze lk:=v;
v=v+1; //v:=v+1;

N(k)=0; //N:=N\{k};
for i=1:n,
vorg(i)=vorg(i)-mat(k,i); //vorg(j):=vorg(j)-akj f?r alle j aus N
end,
weiter2=-1; //gehe zu schritt 2
else,
k=k+1;
end,
end, //zu Schritt 2:
if (k>n) then, //kein k aus N ohne Vorg?nger, Digraph enth?lt Zyklen
disp('Graph enth?lt Zyklen!');
weiter2=-1;
weiter=-1;
end,
end,
if v>n then, //falls v=n terminiere, Digraph ist topologisch sortiert.
weiter=-1;
weiter2=-1;
end,
end
g=graph;
g.node_name=nodenamesnew; //neue Knotennamen dem Graph zuordnen
endfunction

///////////////////////////////////////////////////
//**********************************************///
///////////////////////////////////////////////////


//n?chsten Knoten augeben
function [n]=get_next_node(g,lastnode)
names=g.node_name;
if lastnode==g.node_number then,
n=lastnode;
else,
nn=lastnode+1;
for i=1:g.node_number,
t=sscanf(names(i),'%d');
if t==nn then,
n=i;
end,
end,
end,
endfunction

///////////////////////////////////////////////////
//**********************************************///
///////////////////////////////////////////////////


//Knoten nummer zum knoten mit namen number
function [n]=get_node_number(g, number)
names=g.node_name;
for i=1:g.node_number,
t=sscanf(names(i),'%d');
if t==number then,
n=i;
end,
end,
endfunction

///////////////////////////////////////////////////
//**********************************************///
///////////////////////////////////////////////////

//knoten name nummer zum internen knote number
function[n]=get_name_number(g,number)
names=g.node_name;
n=sscanf(names(number),'%d');
endfunction

///////////////////////////////////////////////////
//**********************************************///
///////////////////////////////////////////////////

//gibt die Kantennummer der Kante von i nach j aus
//ben?tigt einen mit top_sort sortierten Graphen
function [n]=finde_kante(g,i,j)
en=edge_number(g);
n=0; //falls keine Kante gefunden
for k=1:en, //alle kanten
t=get_node_number(g,i); //t interne knotennummer zum knoten namen j
if g.head(k)==t,
if g.tail(k)==get_node_number(g,j),
n=k;
end,
end,
end,
endfunction

///////////////////////////////////////////////////
//**********************************************///
///////////////////////////////////////////////////


//gibt die bewertungsmatrix f?r einen bewerteten graphen
// die matrix folgt der nummerierung in dem vektor node_name

function [c]=bew_matrix(g)
mat=zeros(g.node_number,g.node_number);
sk=g.tail;
ek=g.head;
kl=g.edge_length;
//hohe bewertung eintragen, um aufnahme fehlender kanten zu verhindern
highvalue=sum(kl)^2;
for j=1:size(mat,'r'),
for k=1:size(mat,'c'),
mat(j,k)=highvalue;
end,
end,

for i=1:size(sk,'c'),
//disp("Kante: ");
//disp(i);
ts=get_name_number(g,sk(i));
//disp(ts);
te=get_name_number(g,ek(i));
//disp(te);
mat(ts,te)=kl(i);
//disp(kl(i));
end,
c=mat;
endfunction


//////////////////////////////////////////////////
//*********************************************///
//////////////////////////////////////////////////

//prüft die zulässigkeit eines flusses von r nach s in einem netzwerk g
function [ft]=fluss_ok(g,r,s)
	gtmp=g;
	n=gtmp.node_number;
	en=gtmp.edge_number;
	fl=zeros(n,n); //Matrix mit flusgrößen
	insum=zeros(n);
	outsum=zeros(n);
	
	maxin=zeros(n);
	maxout=zeros(n);
	
	
	for i=1:en, //ale kanten durchgehen
		
			
	end,

endfunction

///////////////////////////////////////////////////
//**********************************************///
///////////////////////////////////////////////////

function [f,s,g1]=anfangsfluss(g,r,s)

	en=edge_number(g);
	n=g.node_number;
	//zirkulastionsflüsse einfügen
	gtmp=g;  //entspricht N' im Kurs
	tail=gtmp.tail;
	head=gtmp.head;
		
	for i=1:en,
		sk=tail(i);
		ek=head(i);
		gtmp=add_edge(ek,sk,gtmp);
	end,
	for i=1:en,		//kapazitäten in neuen pfeilen eintragen
		gtmp.edge_min_cap(i+en)=gtmp.edge_min_cap(i);
		gtmp.edge_max_cap(i+en)=gtmp.edge_max_cap(i);
	end,
	
	g2=add_node(gtmp);
	gtmp=g2;
	qNr=n+1;
	pause
	gtmp=add_edge(qNr,1,gtmp); //neuer Pfeil von Knoten q zu Knoten eins -> q Quelle
	g2=add_node(gtmp);
	gtmp=g2;
	tNr=n+2;
	gtmp=add_edge(1,tNr,gtmp); //dasselbe für neue Senke t
	
	//für jeden Knoten menge der Vorgänger bzw. Nachfolger mit Pfeil mit positiver Minimalkapzität
	en=edge_number(gtmp);
	A=zeros(n,n);
	B=zeros(n,n);
	for i=1:en
		s=gtmp.head(i);
		e=gtmp.tail(i);
		if(gtmp.edge_min_cap(i)>0),
			A(s,e)=1;		
			B(e,s)=1;
		end,		
	end, //Menge Vorg. / Nachf. mit pos. MinKap
	A=full(A);
	sumA=sum(A,'r');  //enthält anzahl der elemente in A für knoten i
	for i=1:n,
		if(sumA(i)>0),		//Ai nicht leer?
			gtmp=add_edge(qNr,i,gtmp);
			nk=finde_kante(gtmp,qNr,i);
			capsum=0;	
			for k=1:n,
								//neue maximalkapazität berechnen
				if(A(i,k)>0),				//k in Ai?
					ak=finde_kante(gtmp,i,k);
					capsum=capsum+gtmp.edge_min_cap(ak);
				end,
			end,
			gtmp.edge_max_cap(nk)=capsum; 		//summe der minimalkapazitäten
		end,		
	end,

	sumB=sum(B,'r');  //enthält anzahl der elemente in A für knoten i
	for i=1:n,
		if(sumB(i)>0),		//Ai nicht leer?
			gtmp=add_edge(tNr,i,gtmp);
			nk=finde_kante(gtmp,tNr,i);
			capsum=0;	
			for k=1:n,							//neue maximalkapazität berechnen
				if(B(i,k)>0),				//k in Ai?
					ak=finde_kante(gtmp,i,k);
					capsum=capsum+gtmp.edge_min_cap(ak);
				end,
			end,
			gtmp.edge_max_cap(nk)=capsum; 		//summe der minimalkapazitäten
		end,		
	end,


endfunction

///////////////////////////////////////////////////
//**********************************************///
///////////////////////////////////////////////////

//maximalflüsse nach ford_fulkerson
//im graphen g müssen kapazitäten angegeben sein

function [f,s,g1]=Ford_Fulkerson(g,r,s)
		

endfunction


///////////////////////////////////////////////////
//**********************************************///
///////////////////////////////////////////////////

function [g1,d,q]=triple(g1)
	g=g1;
	n=g.node_number;

	C=bew_matrix(g);		//Bewertungsmatrix
	
	highvalue=0;				//wert für unendlich in bewertungsmatrix
	for i=1:n,
		for j=1:n,
			if(C(i,j)>highvalue),
				highvalue=C(i,j);
			end,
		end,
	end,
	
	//Schritt 1:
	d=C;		//aktuelle diastanz matrix	
	q=zeros(n,n);
	for i=1:n,
		d(i,i)=0;
		C(i,i)=0;
		for j=1:n,			
			if(C(i,j)<highvalue),
				q(i,j)=i;
			end,			
		end,
	end,

	//Schritt 2:
	
	for v=1:n,
		for i=1:n,
			if(d(i,v)<highvalue),
				for j=1:n,
					if((d(i,v)+d(v,j))<d(i,j)),
						d(i,j)=d(i,v)+d(v,j);
						q(i,j)=q(v,j);
						end,
				end,
			end,
		end,
	end,

endfunction


///////////////////////////////////////////////////
//**********************************************///
///////////////////////////////////////////////////
//k?rzeste Wege nach Bellman
//vector l enth?lt dir L?nge zum Knoten la
//vector w gibt den direkten vorg?nger zum knotel wa an
//der graph g1 enth?lt blau eingezeichnet die k?rzesten wege
function [l,w,g1]=bellman(g,i)
g1=top_sort(g);
n=g1.node_number;
a=i; //a startknoten

nachf=zeros(n); //anzahl nachfolger f?r jeden knoten i
for i=a:n,
t=get_node_number(g1,i);
s=successors(t,g1);
nachf(i)=size(s,'r');,
end,

M=zeros(n,1);

vorg=zeros(n,1); //vorg?nger auf null
for i=1:n,
vorg(n)=0;
end,

c=bew_matrix(g1);
mvalue=max(c);

d=zeros(n,1);
q=zeros(n,1);
for i=1:n,
d(i)=mvalue; //maximalwert der bewertungsmatrix
q(i)=0;
end,

d(a)=0;
q(a)=a;

//disp("eingabedaten berechnet...");
//pause

//berechnung eingangsgrade
for i=a:n, //alle knoten
t=get_node_number(g1,i); //interne knotennr.
s=successors(t,g1); //nachfolger bestimmen
if (size(s,'c')>0),
for j=1:size(s,'c'), //f?r jeden nachfolger
k=get_name_number(g1,s(j)); //nummer im knotennamen
vorg(k)=vorg(k)+1; //f?r diesen anzahl der vorg?nger um eins erh?hen
end,
end,
end,

vorg(a)=0; //a als Startknoten hat keine vorg?nger
for i=a:n, //Menge der Knoten ohne Vorg?nger
if vorg(i)==0,
M(i)=1;
end,
end,

if (sum(M)==0),
disp("Fehler: Graph enhält zyklen!");
abort;
end,

//disp("eingangsgrade berechnet...");
//pause;

//3. hauptschritt
weiter=1;
while weiter>0,
k=1; //irgendein k aus M
weiter2=1;
while weiter2>0,
if(M(k)==1),
weiter2=-1;
else,
if(k<size(M,'r')),
k=k+1;
else,
weiter2=-1;
weiter1=-1;
end,
end,
end,

//disp("aktueller Knoten ermittelt");
//pause

if(k>=size(M)),
weiter=-1;
else,

M(k)=0; //M=M\k
t=get_node_number(g1,k);
s=successors(t,g1);
//disp("nachfolger ermittelt");
//disp(s)
for j=1:size(s,'c');
l=get_name_number(g1,s(j));
vorg(l)=vorg(l)-1;
if(vorg(l)==0),
M(l)=1; end,

if((d(k)+c(k,l))<d(l)),
d(l)=d(k)+c(k,l);
q(l)=k;
end,
//pause
end,
end,
end, //weiter

l=d;
w=q;

//k?rzeste Wege blau zeichen
en=edge_number(g1);
cols=ones(en,1); //schwarze farbe f?r alle
cols=cols';

for i=a:size(w,'r'), //alle knoten ncheinenander
if w(i)==0,
//kein weg
else,
for j=1:en, //alle kanten
t=get_node_number(g1,i); //t interne knotennummer zum knoten namen j
if g1.head(j)==t,
if g1.tail(j)==get_node_number(g1,w(i)),
cols(j)=2;
end,
end,
end,
end,
end,

g1.edge_color=cols;

endfunction

///////////////////////////////////////////////////
//**********************************************///
///////////////////////////////////////////////////


//berechnet die k?rzesten Wege vom Startknoten a
//mit dem Algorthmus von Dijkstra
function [l,w,g1]=dijkstra(g,a)

//g1=top_sort(g); //topologisches Sortieren
g1=g;
n=node_number(g1);
C=bew_matrix(g1);
highvalue=max(C)^2; //f?r unendlich

nachf=zeros(n,1); //anzahl nachfolger f?r jeden knoten i
for i=a:n,
t=get_node_number(g1,i);
s=successors(t,g1);
nachf(i)=size(s,'r');,
end,

//Nachfolger von a bestimmen
t=get_node_number(g1,a); //interne knotennr t
s=successors(t,g1); //nachfolger von t
nf=zeros(size(s,2),1);
for i=1:size(s,2),
nf(i)=get_name_number(g1,s(i)); //namen der nachfolger eintragen
end

L=zeros(n,1);
M=zeros(n,1);
d=zeros(n,1);
q=zeros(n,1);

for i=1:n,
d(i)=highvalue; //dj=unendenlich
end,
d(a)=0; //da=0,
for i=1:size(nf,1),
d(nf(i))=C(a,nf(i)); //f?r nachfogler von a d=c(a,i)
q(nf(i))=a;
M(nf(i))=1;
end,



//Schritt 2:


weiter=1;
while weiter>0,
//k aus M finden, mit min dk

if (sum(M)>0), //Noch Knoten in M?
jinM=zeros(sum(M),1);
p=1; //aktuelle position in jinM
for i=1:n,
if(M(i)==1),
jinM(p)=i;
p=p+1;
end,
end,
//min in vektor d
mind=d(jinM(1));
k=jinM(1);
for i=2:size(jinM,1),
if(mind>d(jinM(i))),
mind=d(jinM(i));
k=jinM(i); //knoten k aus M mit min b
end,
end,

L(k)=1;
//Nachfolger von k bestimmen
t=get_node_number(g1,k); //interne knotennr t
s=successors(t,g1); //nachfolger von t
nf=zeros(size(s,2),1);
for i=1:size(s,2),
nf(i)=get_name_number(g1,s(i)); //namen der nachfolger eintragen
end
for i=1:size(nf,1), //schritt drei
M(nf(i))=1; //nachfolger in Menge M aufnehmen

if (d(nf(i))>(d(k)+C(k,nf(i)))), //l?nge ?berpr?fen
d(nf(i))=d(k)+C(k,nf(i)); //ggef. neue L?nge setzen
q(nf(i))=k; //neuer wegpunkt
end,
end,
for i=1:size(M,1),
if(M(i)==1),
M(i)=M(i)-L(i); //M ohne L
end,
end,

else
weiter=-1; //keine Knoten mehr in M
end,
end,

l=d;
w=q;

en=edge_number(g1);
cols=ones(en,1); //schwarze farbe f?r alle
cols=cols';

for i=1:size(w,'r'), //alle knoten ncheinenander
if w(i)==0,
//kein weg
else,
for j=1:en, //alle kanten
t=get_node_number(g1,i); //t interne knotennummer zum knoten namen j
if g1.head(j)==t,
if g1.tail(j)==get_node_number(g1,w(i)),
cols(j)=2;
end,
end,
end,
end,
end,

g1.edge_color=cols;


endfunction
///////////////////////////////////////////////////
//**********************************************///
///////////////////////////////////////////////////

function [l,w,g1]=ford(g,a)

g1=g;
n=g.node_number;
C=bew_matrix(g1);
highvalue=max(C);

M=zeros(n,1);
L=zeros(n,1);
d=zeros(n,1);
q=zeros(n,1);

for i=1:size(d,1),
	d(i)=highvalue;
end,

//Nachfolger von a bestimmen
	t=get_node_number(g1,a); //interne knotennr t
	s=successors(t,g1); //nachfolger von t
	nf=zeros(size(s,2),1);
	for i=1:size(s,2),
		nf(i)=get_name_number(g1,s(i)); //namen der nachfolger eintragen
	end,

for i=1:size(nf,1),
	d(nf(i))=C(a,nf(i));
	q(nf(i))=a;
	L(nf(i))=1;
end,

//schritt 2:

weiter=1;
while weiter>0,
	if (sum(L)>0), //Noch Knoten in M?
		kinL=zeros(sum(L),1);
		p=1; //aktuelle position in jinM
	for i=1:n,
		if(L(i)==1),
			kinL(p)=i;
			p=p+1;
		end,
	end,
	
	for j=1:size(kinL,1),	
	k=kinL(j);
	t=get_node_number(g1,k); //interne knotennr t
	s=successors(t,g1); //nachfolger von t
	nf=zeros(size(s,2),1);
	for i=1:size(s,2),
		nf(i)=get_name_number(g1,s(i)); //namen der nachfolger eintragen
	end,

	for i=1:size(nf,1),
		if ((d(k)+C(k,nf(i)))<d(nf(i))),
			d(nf(i))=d(k)+C(k,nf(i));
			q(nf(i))=k;
			M(nf(i))=1;
		end,
	end,
	
	end,  //jedes k in L
	
	if (sum(M)==0),
		weiter=-1;
	else,
		L=M;
		M=zeros(n,1);
	end,
	else, //L ist leer
	weiter=-1;
	end,
end,

//Kanten farblich markieren

l=d;
w=q;

en=edge_number(g1);
cols=ones(en,1); //schwarze farbe f?r alle
cols=cols';

for i=1:size(w,'r'), //alle knoten ncheinenander
	if w(i)==0,
	//kein weg
	else,
		for j=1:en, //alle kanten
			t=get_node_number(g1,i); //t interne knotennummer zum knoten namen j
				if g1.head(j)==t,
					if g1.tail(j)==get_node_number(g1,w(i)),
						cols(j)=2;
					end,
				end,
		end,
	end,
end,

g1.edge_color=cols;

endfunction


///////////////////////////////////////////////////
//**********************************************///
///////////////////////////////////////////////////

// minimal ger?st nach Algorithmus von Prim
function [g1]=min_prim(g)
g1=top_sort(g);
n=g1.node_number; //anzahl der Knoten
C=bew_matrix(g1); //bewertungsmatrix

// Initialisierung
L=zeros(n,1);
L(1)=1;
l=ones(n,1);
b=zeros(n,1);
for i=1:n,
b(i)=C(1,i);
end,
M=ones(n,1);
M(1)=0;
I=zeros(n,1);
Izaehler=0;

	weiter=1; //abbruchkriterium

	while weiter>0,
//finde k aus M mit bk =min bj mit j aus M
		k=1;
		if (sum(M)>0), //Noch Knoten in M?
		Izaehler=Izaehler+1; //aktuelle Position in Vektor I
		jinM=zeros(sum(M),1);
		p=1; //aktuelle position in jinM
		for i=1:n,
			if(M(i)==1),
				jinM(p)=i;
				p=p+1;
			end,
		end,

//min in b
		minb=b(jinM(1));
		k=jinM(1);
		for i=2:size(jinM,1),
			if(minb>b(jinM(i))),
				minb=b(jinM(i));
				k=jinM(i); //knoten k aus M mit min b
			end,
		end,

		L(k)=1; // L:=L und {k}
		M(k)=0; //M:= M ohne {k}

//kante von lk und k finden und in I eintragen
		e=finde_kante(g1,k,l(k));
		I(Izaehler)=e;
//schritt 3:
		if (sum(M)>0),
			jinM=zeros(sum(M),1);
			p=1; //aktuelle position in jinM
			for i=1:n,
				if(M(i)==1),
					jinM(p)=i;
					p=p+1;
				end,
			end,
			for i=1:size(jinM,1),
				if(C(k,jinM(i))<b(jinM(i))),
					b(jinM(i))=C(k,jinM(i));
					l(jinM(i))=k;
				end,
			end,
		end,
		else, //keine Knoten in M:
			weiter=-1;
		end,
	end,

//Kanten I farblich makieren

	cols=ones(edge_number(g1),1);
	for i=1:size(I,1),
		if(I(i)>0),
			cols(I(i))=2;
		end,
	end,
	disp(cols);
	g.edge_color=cols';
endfunction


///////////////////////////////////////////////////
//**********************************************///
///////////////////////////////////////////////////

//Funktionen die verschiedene Graphen erzeugen
function [g1]=einfacher_graph()
	g1=make_graph('Einfacher Graph',1,4,[2 3 4 4],[1 1 2 3]);
	g1('node_x')=[100 150 150 200];
	g1('node_y')=[100 50 150 100];
	g1.edge_length=[10 20 15 10];
endfunction

function [g]=kleiner_graph()
	g=make_graph('Kleiner Graph',1,8,[8 8 8 7 6 6 5 4 3 2],[2 7 6 5 3 4 4 1 1 3]);
	g('node_x')=[10 200 100 100 150 200 200 250];
	g('node_y')=[100 0 100 200 250 100 200 100];
	g.edge_length=[4 3 5 4 3 2 1 4 5 6];
endfunction

function [g]=graph1()
	g=make_graph('Graph1',1,5,[1 1 2 2 3 3 4 ],[2 3 3 4 4 5 5]);
	g('node_x')=[100 250 250 400 400];
	g('node_y')=[100 0 200 0 200];
	g.edge_length=[3 5 2 3 4 2 6];
endfunction

function [g]=graph2()
	g=make_graph('Graph1',1,5,[1 1 2 2 3 3 4 2 3 3 4 4 5 5],[2 3 3 4 4 5 5 1 1 2 2 3 3 4]);
	g('node_x')=[100 250 250 400 400];
	g('node_y')=[100 0 200 0 200];
	g.edge_length=[3 5 2 3 4 2 6 3 5 2 3 4 2 6];
endfunction

function [g]=simple_net()
g=make_graph('Net1',1,4,[1 1 2 3],[2 3 4 4]);
g.edge_min_cap=[1 2 2 3];
g.edge_max_cap=[3 4 3 4];
endfunction



function[g]=aufg_1_14()
g=make_graph('Übungsaufgabe 1.14',1,6,[1 2 2 2 3 4 4 4 5 5],[3 1 3 4 5 2 5 6 3 6]);
g.edge_length=[4 2 0 -2 -2 1 1 2 -2 3];
g('node_x')=[100 200 200 300 300 400];
g('node_y')=[100 200 0 200 0 100];
endfunction

function [g]=bsp_3_1()
g=make_graph('Beispiel 3.1',0,7,[1 1 2 2 2 3 3 4 5 5 6],[2 3 3 4 5 5 6 5 6 7 7]);
g.edge_length=[1 2 3 10 4 3 5 6 0 7 8];
g('node_x')=[100 200 200 300 400 400 500];
g('node_y')=[100 200 0 300 200 0 100];
endfunction

function [g]=aufg_3_2()
g=make_graph('Übungsaufgabe 3.2',1,10,[1 2 2 2 3 3 4 5 5 5 5 6 6 6 8 8 9 9 9],[2 3 4 5 1 5 5 3 9 7 8 3 5 9 7 10 5 8 10]);
g.edge_length=[4 -1 5 2 3 6 0 -4 2 3 4 5 0 -3 1 5 0 1 0];
g('node_x')=[100 200 200 250 300 300 350 400 400 500];
g('node_y')=[50 100 0 200 100 0 200 100 0 50];
endfunction

function [g]=bsp_3_2()
g=make_graph('Beispiel 3.2',1,8,[1 1 1 2 2 2 3 3 3 4 4 6 6 7],[5 2 3 5 4 3 4 6 7 5 6 7 8 8]);
g.edge_length=[2 -3 0 6 2 4 5 7 -1 3 -2 3 5 4];
g('node_x')=[0 100 200 200 0 400 400 600];
g('node_y')=[0 100 0 200 200 200 0 100];
endfunction

function[g]=bsp_3_3()
g=make_graph('Beispiel 3.3',1,5,[1 1 2 2 3 3 3 4 4],[2 5 3 4 1 2 5 3 5]);
g.edge_length=[1 2 4 2 2 1 10 1 10];
g('node_x')=[100 0 200 400 300];
g('node_y')=[200 0 100 0 200];
endfunction


function [g]=aufg_3_4()
g=make_graph('Übungsaufgabe 3.4',0,7,[1 1 1 1 2 2 3 3 4 4 4 5 6],[5 4 3 2 4 3 4 6 5 7 6 6 7]);
g.edge_length=[1 5 5 2 4 5 3 3 7 3 2 3 6];
g('node_x')=[100 0 100 200 300 300 250];
g('node_y')=[200 100 0 100 200 0 100];
endfunction


function[g]=bsp_3_4()
g=make_graph('Beispiel 3.4',1,5,[1 1 1 2 3 3 4 5 5],[2 3 4 3 2 4 5 2 3]);
g.edge_length=[10 9 4 4 -1 -6 3 3 4];
g('node_x')=[100 0 200 300 400];
g('node_y')=[200 0 100 200 0];
endfunction


function [g]=bsp_3_5()
g=make_graph('Beispiel 3.5',1,5,[1 1 2 2 3 3 3 3 4 5],[2 3 4 5 1 5 4 2 5 1]);
g.edge_length=[15 7 2 10 8 12 9 6 3 5];
g('node_x')=[0 200 0 200 100];
g('node_y')=[200 200 0 0 100];
endfunction

function [g]=bsp_3_6()
g=make_graph('Beispiel 3.6',1,4,[1 2 2 3 3 4],[2 3 4 1 4 2]);
g.edge_length=[2 -4 3 1 4 0];
g('node_x')=[100 200 100 200];
g('node_y')=[100 100 0 0];
endfunction


function [g]=aufg_3_6()																																	
g=make_graph('Übungsaufgabe 3.6',1,5,[1 1 1 1 2 2 2 3 3 3 3 4 4 4 4 5 5],[2 4 3 5 1 3 5 1 2 5 4 2 1 3 5 2 4]);
g.edge_length=[2 4 2 2 3 4 2 2 5 3 1 3 3 1 1 1 1];			
g('node_x')=[100 400 100 400 250];
g('node_y')=[300 300 0 0 150];
endfunction
